/*
	$Id $

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.AnnotationTypeCategory;
import net.sf.basedb.core.BioPlateType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

/**
	Some utility functions for annotations.
	@author nicklas
	@since 3.5
*/
public class AnnotationUtil 
{

	/**
		Load information about categories and subtypes of an annotation type that
		are useful for deciding about inheriting annotations from parent items,
		The result is returned as a JSON object intended to be used by the
		"Batch inherit annotations" feature in the web interface. 
		See: common/annotations/batch_inherit.jsp (and .js)
	*/
	@SuppressWarnings("unchecked")
	public static JSONObject loadJsonForBatchInherit(DbControl dc, AnnotationType at)
	{
		JSONObject json = new JSONObject();
		json.put("id", at.getId());
		json.put("name", at.getName());
		if (at.getDisableInheritance())
		{
			json.put("disableInheritance", 1);
			return json;
		}
		// Load categories for the annotation type
		ItemQuery<AnnotationTypeCategory> categoryQuery = AnnotationTypeCategory.getQuery();
		categoryQuery.include(Include.MINE, Include.OTHERS, Include.SHARED, Include.IN_PROJECT);
		categoryQuery.join(Hql.innerJoin("annotationTypes", "atp"));
		categoryQuery.restrict(Restrictions.eq(Hql.alias("atp"), Hql.entity(at)));
		categoryQuery.order(Orders.asc(Hql.property("name")));
		
		JSONArray jsonCategories = new JSONArray();
		Set<String> names = new HashSet<String>();
		for (AnnotationTypeCategory cat : categoryQuery.list(dc))
		{
			JSONObject jsonCat = new JSONObject();
			jsonCat.put("id", cat.getId());
			jsonCat.put("name", cat.getName());
			names.add(cat.getName());
			jsonCategories.add(jsonCat);
		}
		json.put("categories", jsonCategories);
		
		// Load subtypes for item types matching items types from the annotation type
		ItemQuery<ItemSubtype> subtypeQuery = ItemSubtype.getQuery(null);
		subtypeQuery.order(Orders.asc(Hql.property("name")));
		subtypeQuery.restrict(Restrictions.in(Hql.property("itemType"), Expressions.parameter("itemTypes")));
		Set<Integer> itemTypes = new HashSet<Integer>();
		boolean hasBioPlateAnnotation = false;
		for (Item itemType : at.getEnabledItems())
		{
			if (itemType == Item.BIOPLATE) hasBioPlateAnnotation = true;
			itemTypes.add(itemType.getValue());
		}
		subtypeQuery.setParameter("itemTypes", itemTypes, Type.INT);
		JSONArray jsonSubtypes = new JSONArray();
		boolean hasDefault = false;
		for (ItemSubtype subtype : subtypeQuery.list(dc))
		{
			JSONObject jsonSubtype = new JSONObject();
			jsonSubtype.put("id", subtype.getId());
			jsonSubtype.put("name", subtype.getName());
			jsonSubtype.put("itemType", subtype.getMainItemType().name());
			if (!hasDefault && names.contains(subtype.getName()))
			{
				jsonSubtype.put("selected", 1);
				hasDefault = true;
			}
			jsonSubtypes.add(jsonSubtype);
		}
		json.put("subtypes", jsonSubtypes);
		
		if (hasBioPlateAnnotation)
		{
			// Load bioplate types linked with subtypes having the same name as the categories
			ItemQuery<BioPlateType> bioPlateTypeQuery = BioPlateType.getQuery();
			bioPlateTypeQuery.join(Hql.innerJoin("itemSubtype", "st"));
			bioPlateTypeQuery.restrict(Restrictions.in(Hql.property("st", "name"), Expressions.parameter("categories")));
			bioPlateTypeQuery.order(Orders.asc(Hql.property("name")));
			bioPlateTypeQuery.setParameter("categories", names, Type.STRING);
			
			JSONArray jsonPlateTypes = new JSONArray();
			for (BioPlateType plateType : bioPlateTypeQuery.list(dc))
			{
				ItemSubtype subtype = plateType.getItemSubtype();
				
				JSONObject jsonPlateType = new JSONObject();
				jsonPlateType.put("id", plateType.getId());
				jsonPlateType.put("name", plateType.getName());
				
				JSONObject jsonSubtype = new JSONObject();
				jsonSubtype.put("id", subtype.getId());
				jsonSubtype.put("name", subtype.getName());
				jsonSubtype.put("itemType", subtype.getMainItemType().name());
				jsonPlateType.put("subtype", jsonSubtype);

				jsonPlateTypes.add(jsonPlateType);
			}
			json.put("plateTypes", jsonPlateTypes);
		}

		return json;
	}
	
}
