/**
	$Id: ServletWrapper.java 6131 2012-09-14 09:13:06Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.iterators.IteratorEnumeration;

/**
	A wrapper class for servlets defined by extensions. This wrapper holds
	initialisation parameters and the servlet context that a servlet
	needs. Each request to a servlet will create a new instance of the
	servlet, followed by calls to {@link Servlet#init(ServletConfig)},
	{@link Servlet#service(javax.servlet.ServletRequest, javax.servlet.ServletResponse)}
	and {@link Servlet#destroy()}.

	@author nicklas
	@version 2.7
	@base.modified $Date: 2012-09-14 11:13:06 +0200 (fr, 14 sep 2012) $
*/
public class ServletWrapper
	implements ServletConfig
{

	private final Class<? extends Servlet> servletClass;
	private final String name;
	private ServletContext context;
	private final Map<String, String> parameters;
	
	public ServletWrapper(Class<? extends Servlet> servletClass, String name)
	{
		this.servletClass = servletClass;
		this.name = name;
		this.parameters = new HashMap<String, String>();
	}

	/**
		Service a request to the wrapped servlet. A new servlet instance
		is created for each request.
		
		@param request The request parameters
		@param response The response object
		@throws ServletException
		@throws IOException
	*/
	public void service(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		try
		{
			Servlet servlet = servletClass.newInstance();
			servlet.init(this);
			servlet.service(request, response);
			servlet.destroy();
		}
		catch (IllegalAccessException ex)
		{
			throw new ServletException(ex);
		}
		catch (InstantiationException ex)
		{
			throw new ServletException(ex);
		}
	}
	
	/**
		Set the value of a servlet parameter.
		@param name The name of the parameter
		@param value The value of the parameter
	*/
	void setParameter(String name, String value)
	{
		parameters.put(name, value);
	}
	
	/**
		Set the servlet context from the web application.
	*/
	void setServletContext(ServletContext context)
	{
		this.context = context;
	}

	/*
		From the ServletConfig interface
		--------------------------------
	*/
	@Override
	public String getInitParameter(String name)
	{
		return parameters.get(name);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Enumeration<String> getInitParameterNames()
	{
		return (Enumeration<String>)new IteratorEnumeration(parameters.keySet().iterator());
	}

	@Override
	public ServletContext getServletContext()
	{
		return context;
	}

	@Override
	public String getServletName()
	{
		return name;
	}
	// ---------------------------------------------------

}
