/*
	$Id: AnnotationRestriction.java 6686 2015-01-20 12:55:44Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.hibernate.TypeWrapper;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.Restriction;
import net.sf.basedb.util.EqualsHelper;

import org.hibernate.SQLQuery;
import org.hibernate.dialect.Dialect;

import java.util.Collection;
import java.util.List;

/**
	Base class for creating restrictions based on annotations.
	Subclasses must implent the {@link #getRestrictionSql(Query, DbControl)}
	and {@link #setRestrictionParameters(SQLQuery)}
	method.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.query.Annotations
	@base.modified $Date: 2015-01-20 13:55:44 +0100 (ti, 20 jan 2015) $
*/
abstract class AnnotationRestriction
	implements Restriction
{
	/**
		Log all SQL statements.
	*/
	private static final org.slf4j.Logger logSql = 
		org.slf4j.LoggerFactory.getLogger("net.sf.basedb.core.query.ql");
	
	/**
		So we don't always have to call logSql.debug()
	*/
	private static final boolean debugSqlEnabled = logSql.isDebugEnabled();
	
	final Type valueType;
	final int annotationTypeId;
	final boolean includePrimary;
	final boolean includeInheriting;
	final String alias;

	/**
		Create a new annotation restriction for an annotation type.
		@param annotationType The annotation type to use in the query
	*/
	AnnotationRestriction(String alias, AnnotationType annotationType, boolean includePrimary, boolean includeInheriting)
		throws InvalidDataException
	{
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		if (!includePrimary && !includeInheriting) 
		{
			throw new InvalidDataException("Both includePrimary and includeInheriting parameters are FALSE");
		}
		this.annotationTypeId = annotationType.getId();
		this.valueType = annotationType.getValueType();
		this.includePrimary = includePrimary;
		this.includeInheriting = includeInheriting;
		this.alias = alias;
	}

	/**
		Create a new annotation restriction given the id and value type of
		an annotation. It is not verified that there exists an annotation
		type with the specified id or that it's value type is the one specified.
		The only thing that will happen is that the restriction will not match
		any annotations.
		@param annotationTypeId The id of the annotation type
		@param valueType The type of values for annotations
	*/
	AnnotationRestriction(String alias, int annotationTypeId, Type valueType, boolean includePrimary, boolean includeInheriting)
		throws InvalidDataException
	{
		if (!includePrimary && !includeInheriting) 
		{
			throw new InvalidDataException("Both includePrimary and includeInheriting parameters are FALSE");
		}
		this.annotationTypeId = annotationTypeId;
		this.valueType = valueType;
		this.includePrimary = includePrimary;
		this.includeInheriting = includeInheriting;
		this.alias = alias;
	}

	/*
		From the Restriction interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		SessionControl sc = dc.getSessionControl();
		String valueTable = valueType == null ? null : valueType.getTableName();
		
		// Verify that the annotation type exists
		try
		{
			AnnotationType.getById(dc, annotationTypeId);
		}
		catch (ItemNotFoundException ex)
		{
			return "1=1";
		}
		
		Dialect dialect = HibernateUtil.getDialect();
		char oq = dialect.openQuote();
		char cq = dialect.closeQuote();
		
		String valueRestriction = getRestrictionSql(query, dc);
		StringBuilder sb = new StringBuilder();
		/*
			SELECT a.annotationset_id, [a.id]
			FROM Annotations a [, valueTable v]
			WHERE a.annotationtype_id = :annotationType
			[AND a.value_id = v.id [AND valueRestriction]]
		*/
		sb.append("SELECT ").append("a.").append(oq).append("annotationset_id").append(cq);
		if (includeInheriting)
		{
			sb.append(", a.").append(oq).append("id").append(cq);
		}
		sb.append(" FROM ").append(oq).append("Annotations").append(cq).append(" a");
		if (valueTable != null)
		{
			sb.append(", ").append(oq).append(valueTable).append(cq).append(" v");
		}
		sb.append(" WHERE ");
		sb.append("a.").append(oq).append("annotationtype_id").append(cq).append(" = :annotationType");
		if (valueTable != null)
		{
			sb.append(" AND a.").append(oq).append("value_id").append(cq);
			sb.append(" = v.").append(oq).append("id").append(cq);
			if (valueRestriction != null)
			{
				sb.append(" AND ").append(valueRestriction);
			}
		}
		String sql = sb.toString();

		org.hibernate.Session session = dc.getHibernateSession();
		SQLQuery sqlQuery = HibernateUtil.createSqlQuery(session, sql);
		sqlQuery.addScalar("annotationset_id", TypeWrapper.INTEGER.getHibernateType());
		if (includeInheriting) 
		{
			sqlQuery.addScalar("id", TypeWrapper.INTEGER.getHibernateType());
		}
		sqlQuery.setInteger("annotationType", annotationTypeId);
		setRestrictionParameters(sqlQuery);
		
		String annotationSetIds = null;
		if (includeInheriting)
		{
			if (debugSqlEnabled) logSql.debug("Executing annotation query: " + sql);
			List<Object[]> twoIds = HibernateUtil.loadList(Object[].class, sqlQuery, sc);
			StringBuilder annotationSets = new StringBuilder("0");
			StringBuilder annotations = new StringBuilder("0");
			for (Object[] id : twoIds)
			{
				annotationSets.append(",").append(id[0]);
				annotations.append(",").append(id[1]);
			}
			annotationSetIds = annotationSets.toString();
			String annotationIds = annotations.toString();

			if (twoIds.size() > 0)
			{
				sql = "SELECT ia."+oq+"annotationset_id"+cq+" AS sid"+
					" FROM "+oq+"InheritedAnnotations"+cq+" ia"+
					" WHERE ia."+oq+"annotation_id"+cq+" IN ("+annotationIds+")"+
					" UNION"+
					" SELECT ias."+oq+"annotationset_id"+cq+" AS sid"+
					" FROM "+oq+"InheritedAnnotationSets"+cq+" ias"+
					" WHERE ias."+oq+"inherited_id"+cq+" IN ("+annotationSetIds+")";
				
				sqlQuery = HibernateUtil.createSqlQuery(session, sql);
				sqlQuery.addScalar("sid", TypeWrapper.INTEGER.getHibernateType());
				if (debugSqlEnabled) logSql.debug("Executing annotation query: " + sql);
				List<Integer> ids = HibernateUtil.loadList(Integer.class, sqlQuery, sc);
				annotationSets = new StringBuilder(includePrimary ? annotationSetIds : "0");
				for (Integer id : ids)
				{
					annotationSets.append(",").append(id);
				}
				annotationSetIds = annotationSets.toString();
			}
		}
		else
		{
			if (debugSqlEnabled) logSql.debug("Executing annotation query: " + sql);
			List<Integer> ids = HibernateUtil.loadList(Integer.class, sqlQuery, sc);
			StringBuilder annotationSets = new StringBuilder("0");
			for (Integer id : ids)
			{
				annotationSets.append(",").append(id);
			}
			annotationSetIds = annotationSets.toString();
		}
		boolean notAmongSelected = notAmongSelected();
		String theAlias = alias != null ? alias : query.getRootAlias();
		StringBuilder hql = new StringBuilder("(");
		hql.append(theAlias).append(".annotationSet");
		if (notAmongSelected) hql.append(" NOT");
		hql.append(" IN (").append(annotationSetIds).append(")");
		if (notAmongSelected)
		{
			hql.append(" OR ").append(theAlias).append(".annotationSet IS NULL");
		}
		hql.append(")");
		return hql.toString();
	}
	/**
		@return Always null
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -----------------------------------------

	/**
		Check as many things as possible to try to determine if the other
		object is equal to this object. This method checks if the other instance
		is the same instance as this object (returns true), if the other instance
		is null or not the same class (returns false), if the other instance has
		a different annotation type, alias or inheritance setting (returns false).
		Subclasses should do more checks if null is returned.
		@return TRUE or FALSE if it is certain or null if more checks are needed
	*/
	protected Boolean maybeEquals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		AnnotationRestriction o = (AnnotationRestriction)other;
		if (annotationTypeId != o.annotationTypeId || !EqualsHelper.equals(alias, o.alias) ||
			(includeInheriting != o.includeInheriting))
		{
			return false;
		}
		return null;
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(alias) + annotationTypeId;
	}
	// -------------------------------------------

	
	/**
		Get an SQL fragment that includes the actual restriction.
		Use 'v.value' to access the annotation value. Avoid including
		literal values originating from a user, especially strings, since it may lead 
		to security holes unless the values are checked
		for dangerous characters. Use a parameter placeholder instead. A parameter
		placeholder is a colon (:) followed by a name.
		Example:
		<pre class="code">
v.value = :theValue
</code>
		The <code>setRestrictionParameters</code> method is called to 
		allow the subclass to set the value of the parameters specified
		in the SQL fragment.

		@param query The query object we are about to execute
		@return An SQL fragment, or null if no additional restriction is required
		@see #setRestrictionParameters(SQLQuery)
	*/
	abstract String getRestrictionSql(Query query, DbControl dc)
		throws BaseException;
	
	/**
		Set the value for all parameters defined in the SQL fragment
		returned by {@link #getRestrictionSql(Query, DbControl)}.
		@param query The Hibernate <code>SQLQuery</code> object which is
			used to query the database
	*/
	abstract void setRestrictionParameters(SQLQuery query);
	
	/**
		If the main restriction should match items that had annotations 
		among the selected annotation sets, or not.
		@return FALSE (default) to match the selected annotation sets, TRUE
			to match the non-selected annotation sets
		@since 2.7
	*/
	boolean notAmongSelected()
	{
		return false;
	}
}

