/**
	$Id: BioPlateEventType.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

import net.sf.basedb.core.data.BioPlateEventTypeData;

/**
	The type of a bioplate event defines what happened in the event. A type is 
	usually coupled to a certain functionality in the client application.

	@author Nicklas
	@since 2.17
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class BioPlateEventType
	extends BasicItem<BioPlateEventTypeData>
	implements Nameable, Removable, SystemItem
{

	/**
		The type of item represented by this class.
		@see Item#BIOPLATEEVENTTYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.BIOPLATEEVENTTYPE;

	/**
		The system ID for "Generic" event type.
	*/
	public static final String GENERIC = "net.sf.basedb.core.BioPlateEventType.GENERIC";

	/**
		The system ID for "Place-on-plate" event type.
	*/
	public static final String PLACE_ON_PLATE = "net.sf.basedb.core.BioPlateEventType.PLACE_ON_PLACE";

	/**
		The system ID for "Move" event type.
	*/
	public static final String MOVE = "net.sf.basedb.core.BioPlateEventType.MOVE";

	/**
		The system ID for "Create" event type.
	*/
	public static final String CREATE_BIOMATERIAL = "net.sf.basedb.core.BioPlateEventType.CREATE_BIOMATERIAL";

	/**
		The system ID for "Hybridization" event type.
	*/
	public static final String PHYSICAL_BIOASSAY = "net.sf.basedb.core.BioPlateEventType.PHYSICAL_BIOASSAY";

	
	/**
	 	Creates a new bioplate event type.
	 	 
	  	@param dc DbControl used for database access
	  	@param systemId The system id to give to the event type (can't be changed later)
	  	@return A new BioPlateEventType object
	  	@throws BaseException If anything goes wrong when creating
	*/
	public static BioPlateEventType getNew(DbControl dc, String systemId)
		throws BaseException, InvalidDataException
	{
		BioPlateEventType bp = dc.newItem(BioPlateEventType.class);
		bp.setName("New bioplate event type");
		bp.setSystemId(systemId);
		return bp;
	}

	/**
	 	Gets a BioPlate event type when knowing the item's id
	  	@param dc DbControl used for database access
	  	@param id The item's id.
	  	@return <code>BioPlateEventType</code> object
	  	@throws ItemNotFoundException If no item with the given id was found
	  	@throws PermissionDeniedException If logged in user doesn't have read 
	  		permission to the requested item
	  	@throws BaseException If there is another error.
	*/
	public static BioPlateEventType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		BioPlateEventType eventType = dc.loadItem(BioPlateEventType.class, id);
		if (eventType == null) throw new ItemNotFoundException("BioPlateEventType[id="+id+"]");
		return eventType;
	}
	
	/**
	 	Gets a BioPlate event type when knowing the item's system id
	  	@param dc DbControl used for database access
	  	@param systemId The item's system id.
	  	@return <code>BioPlateEventType</code> object
	  	@throws ItemNotFoundException If no item with the given id was found
	  	@throws PermissionDeniedException If logged in user doesn't have read 
	  		permission to the requested item
	  	@throws BaseException If there is another error.
	*/
	public static BioPlateEventType getById(DbControl dc, String systemId)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		int id = SystemItems.getId(systemId);
		BioPlateEventType eventType = dc.loadItem(BioPlateEventType.class, id);
		if (eventType == null) throw new ItemNotFoundException("BioPlateEventType[systemId="+systemId+"]");
		return eventType;
	}

	
	/**
	 	Gets query that returns bioplate event types.
	  	@return <code>ItemQuery</code> object
	*/
	public static ItemQuery<BioPlateEventType> getQuery()
	{
		return new ItemQuery<BioPlateEventType>(BioPlateEventType.class);
	}
	
	BioPlateEventType(BioPlateEventTypeData data)
	{
		super(data);
	}
	
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}	
	//-------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	@Override
	public String getName()
	{
		return getData().getName();
	}
	@Override
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	@Override
	public String getDescription()
	{
		return getData().getDescription();
	}
	@Override
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	@Override
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	@Override
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	// -------------------------------------------
	/*
	 	From the BasicItem class
	 	--------------------------------------
	 */
	/**
	 	Check if there are any <code>BioPlateEvents</code> using this type.
	*/
	@Override
	public boolean isUsed()
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOPLATEEVENTS_FOR_BIOPLATEEVENTTYPE", "count(*)");
		/*
		 	SELECT {1}
		 	FROM BioPlateEventData bpe
		 	WHERE bpe.eventType = :eventType
		 */
		query.setEntity("eventType", this.getData());
		boolean used = HibernateUtil.loadData(Long.class, query) > 0;
		return used || super.isUsed();
	}
	
	/**
	 	Gets all <code>BioPlates</code> and <code>BioPlateEventDefinitions</code> 
	 	using this type
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		// Bioplates
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOPLATEEVENTS_FOR_BIOPLATEEVENTTYPE", "bpe.id");
		/*
		 	SELECT {1}
		 	FROM BioPlateEventData bpe
		 	WHERE bpe.eventType = :eventType
		*/
		query.setEntity("eventType", this.getData());
		addUsingItems(using, Item.BIOPLATEEVENT, query);
		return using;
	}
	@Override
	void onAfterInsert() 
		throws BaseException
	{
		super.onAfterInsert();
		SystemItems.add(getData());
	}
	//-------------------------------------------

	/*
		From the SystemItem interface
		-------------------------------------------
	*/
	@Override
	public String getSystemId()
	{
		return getData().getSystemId();
	}

	@Override
	public boolean isSystemItem()
	{
		return getSystemId() != null;
	}
	// -------------------------------------------

	void setSystemId(String systemId)
	{
		getData().setSystemId(StringUtil.setNotNullString(systemId, "systemId", MAX_SYSTEM_ID_LENGTH));
	}
}
