/*
	$Id: FilterBatcher.java 6469 2014-06-10 13:52:45Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.hibernate.ExecuteUpdateWork;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Selects;

import java.util.List;
import java.util.LinkedList;
import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
	Batcher class for filtered bioassaysets. A filtered bioassayset is normally
	the result of a filtering transformation and uses the same data cube and
	layer as the source bioassayset to store it's data. This batcher is used
	to specify the column/position coordinates of the original spots that
	remains after the filter has been applied.
	<p>
	Code example: Filter spots where ch1 &lt;= 2 * ch2
	
<pre class="code">
// Get source bioassayset
DbControl dc = ...
BioAssaySet source = ...

// Create new transformation and bioassayset to hold the
// filtered result
Transformation t = source.newTransformation();
dc.saveItem(t);
BioAssaySet filtered = t.newProduct(null, null, true);
dc.saveItem(filtered);

// Create a query that selects the column
// and position of all spots where ch1 &gt; 2 * ch2
DynamicSpotQuery query = source.getSpotData();
query.select(Dynamic.select(VirtualColumn.COLUMN));
query.select(Dynamic.select(VirtualColumn.POSITION));
query.restrict(
   Restriction.gt(
      Dynamic.column(VirtualColumn.channel(1)),
      Expressions.multiply(
         Expressions.integer(2),
         Dynamic.column(VirtualColumn.channel(2))
      )
   )
);

// Execute the query, and use the batcher to insert the result
FilterBatcher batcher = filtered.getFilterBatcher();
DynamicResultIterator result = query.iterate(dc);
int column = result.getIndex(VirtualColumn.COLUMN.getName());
int position = result.getIndex(VirtualColumn.POSITION.getName());
while (result.hasNext())
{
   SqlResult r = result.next();
   batcher.insert(r.getShort(column), r.getInt(position));
}

// Close, save and commit
batcher.close();
dc.commit();
</pre>

	@author Nicklas
	@version 2.0
	@see BioAssaySet#getFilterBatcher()
	@base.modified $Date: 2014-06-10 15:52:45 +0200 (ti, 10 jun 2014) $
*/
public class FilterBatcher
	extends AbstractBatcher
{
	
	/**
		The bioassayset this batcher inserts data for.
	*/
	private final BioAssaySet bioAssaySet;
	
	/**
		The number of queued inserts. Used to check when a flush
		is needed.
	*/
	private int queuedInsertCount = 0;
	
	/**
		The total number of inserts done by this batcher.
	*/
	private int totalInsertCount = 0;
		
	/**
		The number of bytes this batcher has added to the experiment.
	*/
	private long bytes = 0;

	/**
		The number of bytes used by a single row.
	*/
	private final long bytesPerRow;

	/**
		The statement that inserts filter data.
<pre class="code">
INSERT INTO Dynamic#Filter (cube, filter, column, position)
VALUES (cube, filter, ?, ?)
</pre>
	*/
	private PreparedStatement filterSql = null;
	
	/**
	 	The SQL string for the {@link #filterSql} statement.
	*/
	private String filterSqlStatement;

	/**
		Create a new filter batcher for a bioassayset. It is assumed that
		the calling code has checked that the bioassayset is allowed
		to receive more data (ie. it hasn't been committed to the database).
	*/
	FilterBatcher(DbControl dc, BioAssaySet bioAssaySet)
	{
		super();
		this.bioAssaySet = bioAssaySet;
		this.bytesPerRow = VirtualTable.FILTER.getBytesPerRow(bioAssaySet.getVirtualDb());
		setDbControl(dc);
	}

	/*
		From the Batcher interface
		-------------------------------------------
	*/
	@Override
	public void flush()
		throws BaseException
	{
		try
		{
			if (filterSql != null) filterSql.executeBatch();
		}
		catch (SQLException ex)
		{
			logSql.error(filterSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		queuedInsertCount = 0;
	}
	// -------------------------------------------
	/*
		From the AbstractBatcher class
		-------------------------------------------
	*/
	/**
		Close open SQL statements.
	*/
	@Override
	void onBeforeClose()
		throws BaseException
	{
		bioAssaySet.getDataCubeFilter().addBytes(bytes);
		try
		{
			if (filterSql != null) 
			{
				filterSql.close();
				filterSql = null;
			}
		}
		catch (SQLException ex)
		{
			logSql.error(ex.getMessage(), ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
		}
		if (totalInsertCount > 0 && !getDbControl().isRolledBack())
		{
			String catalog = Application.getDynamicCatalog();
			String schema = Application.getDynamicSchema();
			String table = VirtualTable.FILTER.getTableName(bioAssaySet.getVirtualDb());
			analyzeTable(catalog, schema, table);
		}
	}
	// -------------------------------------------

	/**
		Insert a column/position coordinate that passed the filter.
		@param column The column number
		@param position The position number
		@throws BaseException If there is an error
	*/
	public void insert(short column, int position)
		throws BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		try
		{
			if (filterSql == null) buildFilterSql();
			filterSql.setInt(1, column);
			filterSql.setInt(2, position);
			filterSql.addBatch();
			bytes += bytesPerRow;
			totalInsertCount++;
		}
		catch (SQLException ex)
		{
			logSql.error(filterSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		if (++queuedInsertCount == getBatchSize())
		{
			flush();
		}
	}

	/**
		Insert column/position coordinates using a query. The query MUST select data in
		the following order: column, position
		<p>
		Internally this method converts the query into a 
		<code>INSERT INTO ... SELECT ...</code> query, making it very fast,
		since the data for each coordinate doesn't have to be retreived, processed and
		then inserted one row at a time. It is possible to call
		this method multiple times with the same or different query. 
		<p>
		Here is an alternative to the example above:
		<pre class="code">
// Get source bioassayset
DbControl dc = ...
BioAssaySet source = ...

// Create new transformation and bioassayset to hold the
// filtered result
Transformation t = source.newTransformation();
dc.saveItem(t);
BioAssaySet filtered = t.newProduct(null, null, true);
dc.saveItem(filtered);

// Create a query that selects the column
// and position of all spots where ch1 &gt; 2 * ch2
DynamicSpotQuery query = source.getSpotData();
query.select(Dynamic.select(VirtualColumn.COLUMN));
query.select(Dynamic.select(VirtualColumn.POSITION));
query.restrict(
   Restriction.gt(
      Dynamic.column(VirtualColumn.channel(1)),
      Expressions.multiply(
         Expressions.integer(2),
         Dynamic.column(VirtualColumn.channel(2))
      )
   )
);

// Execute the query, and use the batcher to insert the result
FilterBatcher batcher = filtered.getFilterBatcher();
batcher.insert(query);

// Close, save and commit
batcher.close();
dc.commit();
</pre>
		
		@param query The query that selects the data to be inserted
		@return int The number of rows inserted
		@throws BaseException If there is an error
	*/
	public int insert(final AbstractSqlQuery query)
		throws BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		if (query == null) throw new InvalidUseOfNullException("query");
		
		query.resetTemporary();
		
		// Add static values cubeNo and filterNo to the selection
		query.temporarySelect(
			Selects.expression(
				Expressions.integer(bioAssaySet.getDataCubeNo()), 
				VirtualColumn.CUBE.getName()
			)
		);
		query.temporarySelect(
			Selects.expression(
				Expressions.integer(bioAssaySet.getDataCubeFilterNo()), 
				VirtualColumn.FILTER.getName()
			)
		);
		
		String selectSql = query.getMainQuery(getDbControl(), true);
		
		// Holds the names of the parameters in the query in the order they appear
		final List<String> parameterOrder = new LinkedList<String>();
		
		// Parse out named parameters
		selectSql = query.parseParameters(selectSql, parameterOrder);
		String insertSql = buildInsertSelectSql(selectSql);

		int rowsInserted = 0;
		try
		{
			rowsInserted = HibernateUtil.doReturningWork(getDbControl().getHibernateSession(), 
				new ExecuteUpdateWork(getSessionControl(), insertSql)
				{
					@Override
					public void setParameters(PreparedStatement ps)
						throws SQLException
					{
						query.setParameters(ps, parameterOrder, 1);
					}
					
				}
			);
			bytes += rowsInserted * bytesPerRow;
			totalInsertCount += rowsInserted;
		}
		catch (SQLException ex)
		{
			logSql.error(insertSql, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		return rowsInserted;
	}
	
	/**
		Builds the insert SQL statement. Also creates the Dynamic#Filter table
		if it doesn't exists.
		@see BatchUtil#buildInsertSql(VirtualDb, VirtualTable, VirtualColumn[], Object[])
	*/
	private void buildFilterSql()
		throws SQLException, BaseException
	{
		VirtualDb vdb = bioAssaySet.getVirtualDb();
		if (!vdb.hasTable(VirtualTable.FILTER))
		{
			vdb.createTables(VirtualTable.FILTER);
		}
		short cube = bioAssaySet.getDataCubeNo();
		short filter = bioAssaySet.getDataCubeFilterNo();
		filterSqlStatement = BatchUtil.buildInsertSql(
			vdb,
			VirtualTable.FILTER,
			VirtualTable.FILTER.getColumns(vdb), 
			new Object[] {cube, filter}
		);
		filterSql = HibernateUtil.getConnection(getDbControl().getHibernateSession()).prepareStatement(filterSqlStatement);
	}
	
	/**
		Build the INSERT INTO ... SELECT ... statment. Also creates the Dynamic#Filter table
		if it doesn't exists.
		@param selectSql The SELECT part of the query
		@see BatchUtil#buildInsertSelectSql(VirtualDb, VirtualTable, VirtualColumn[], String)
	*/
	private String buildInsertSelectSql(String selectSql)
		throws BaseException
	{
		VirtualDb vdb = bioAssaySet.getVirtualDb();
		if (!vdb.hasTable(VirtualTable.FILTER))
		{
			vdb.createTables(VirtualTable.FILTER);
		}
		String sql = BatchUtil.buildInsertSelectSql(vdb, VirtualTable.FILTER, VirtualTable.FILTER.getColumns(vdb), selectSql);
		return sql;
	}
}
