/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

import net.sf.basedb.core.data.PermissionTemplateData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	A permission template with permissions to users, groups and 
	projects. The template can be assigned to a project ({@link 
	Project#setPermissionTemplate(PermissionTemplate)}) to make new
	items be shared with the permissions from the template when the
	project is the active project.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class PermissionTemplate
	extends CommonItem<PermissionTemplateData>
{
	/**
		The type of item represented by this class.
		@see Item#PERMISSIONTEMPLATE
		@see #getType()
	*/
	public static final Item TYPE = Item.PERMISSIONTEMPLATE;
	
	/**
		Create a new permission template item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@return The new <code>PermissionTemplate</code> item
		@throws BaseException If there is an error
	*/
	public static PermissionTemplate getNew(DbControl dc)
		throws BaseException
	{
		PermissionTemplate pt = dc.newItem(PermissionTemplate.class);
		pt.setName("New permission template");
		return pt;
	}

	/**
		Get a permission template item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>PermissionTemplate</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static PermissionTemplate getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		PermissionTemplate pt = dc.loadItem(PermissionTemplate.class, id);
		if (pt == null) throw new ItemNotFoundException("PermissionTemplate[id="+id+"]");
		return pt;
	}

	/**
		Get a query that returns permission templates.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<PermissionTemplate> getQuery()
	{
		return new ItemQuery<PermissionTemplate>(PermissionTemplate.class);
	}

	PermissionTemplate(PermissionTemplateData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>A {@link Project} uses this template
		</ul>
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PROJECTS_FOR_PERMISSION_TEMPLATE", "count(*)");
		/*
			SELECT {1}
			FROM ProjectData prj
			WHERE prj.permissionTemplate = :permissionTemplate
		*/
		query.setEntity("permissionTemplate", this.getData());
		boolean used = HibernateUtil.loadData(Long.class, query) > 0;
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link Project}:s using this permission template
		<ul>
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Labeled extracts
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_PROJECTS_FOR_PERMISSION_TEMPLATE", "prj.id");
		/*
			SELECT {1}
			FROM ProjectData prj
			WHERE prj.permissionTemplate = :permissionTemplate
		*/
		query.setEntity("permissionTemplate", this.getData());
		addUsingItems(using, Item.PROJECT, query);
		return using;
	}
	// -------------------------------------------

	/**
		Get a query that returns all projects that use this permission template.
	*/
	public ItemQuery<Project> getProjects()
	{
		ItemQuery<Project> query = Project.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("permissionTemplate"), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
}
