/*
	$Id: PlateEventType.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Set;

import net.sf.basedb.core.data.PlateEventTypeData;
import net.sf.basedb.core.data.SharedData;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;

/**
	This class is used together with the <code>PlateType</code> class
	to define event types for plates. A plate may be associated with one
	of each event type.

	@author Nicklas
	@version 2.0
	@see Plate
	@see PlateType
	@see PlateEvent
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class PlateEventType
	extends ChildItem<PlateEventTypeData>
	implements Nameable, Removable
{
	/**
		The type of item represented by this class.
		@see Item#PLATEEVENTTYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.PLATEEVENTTYPE;
	
	/**
		This filter will only return items if the logged in user has
		generic read permission to plate types.
	*/
	private static final QueryRuntimeFilter RUNTIME_FILTER = 
		new QueryRuntimeFilterFactory.ChildFilter(TYPE, Item.PLATETYPE);

	/**
		Create a new <code>PlateEventType</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param plateType The plate type
		@return The new <code>PlateEventType</code> item
		@throws PermissionDeniedException If the loggged in user
			doesn't have write permission for the plate type
		@throws InvalidDataException If the plate type is null
		@throws BaseException If there is an error
	*/
	public static PlateEventType getNew(DbControl dc, PlateType plateType)
		throws InvalidDataException, BaseException
	{
		PlateEventType pet = dc.newItem(PlateEventType.class);
		pet.setName("New plate event type");
		pet.setPlateType(plateType);
		pet.setOrdinal(1);
		return pet;
	}

	/**
		Get a <code>PlateEventType</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>PlateEventType</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static PlateEventType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		PlateEventType pet = dc.loadItem(PlateEventType.class, id);
		if (pet == null) throw new ItemNotFoundException("PlateEventType[id="+id+"]");
		return pet;
	}

	/**
		Get a query configured to retrieve 
		event types for the specified plate type
		
		@param plateType The plate type to retreive event types for, null is allowed if
			the logged in user has generic READ permission for plate types in which case
			all event types will be returned
		@return An {@link ItemQuery} object
		@see PlateType#getEventTypes()
	*/
	public static ItemQuery<PlateEventType> getQuery(PlateType plateType)
	{
		ItemQuery<PlateEventType> query = null;
		if (plateType != null)
		{
			query = new ItemQuery<PlateEventType>(PlateEventType.class, null);
			query.restrictPermanent(
				Restrictions.eq(
					Hql.property("plateType"), 
					Hql.entity(plateType)
				)
			);
		}
		else
		{
			query = new ItemQuery<PlateEventType>(PlateEventType.class, RUNTIME_FILTER);
		}
		return query;
	}

	PlateEventType(PlateEventTypeData plateEventTypeData)
	{
		super(plateEventTypeData);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	@Override
	public String getName()
	{
		return getData().getName();
	}
	@Override
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	@Override
	public String getDescription()
	{
		return getData().getDescription();
	}
	@Override
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	@Override
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	@Override
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>A {@link PlateEvent} of this type exists
		</ul>
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PLATEEVENTS_FOR_PLATEEVENTTYPE", "count(*)");
		/*
			SELECT {1}
			FROM PlateEventData pe
			WHERE pe.plateEventType = :plateEventType
		*/
		query.setEntity("plateEventType", this.getData());
		return HibernateUtil.loadData(Long.class, query) > 0 || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link PlateEvent}:s of this type
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PLATEEVENTS_FOR_PLATEEVENTTYPE", "pe.id");
			/*
				SELECT {1}
				FROM PlateEventData pe
				WHERE pe.plateEventType = :plateEventType
			*/
		query.setEntity("plateEventType", this.getData());
		addUsingItems(using, Item.PLATEEVENT, query);
		return using;
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	@Override
	SharedData getSharedParent()
	{
		return getData().getPlateType();
	}
	// -------------------------------------------

	/**
		Get the {@link PlateType} this event type belongs to.
		@return The <code>PlateType</code> item
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#READ} permission for the plate type
		@throws BaseException If there is another error
	*/
	public PlateType getPlateType()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(PlateType.class, getData().getPlateType());
	}
	/**
		Set the plate type owning this event type.
		@throws PermissionDeniedException If the logged in user
			doesn't have write permission for the plate type
		@throws InvalidDataException If the plate type is null
	*/
	private void setPlateType(PlateType plateType)
		throws PermissionDeniedException, InvalidDataException
	{
		if (plateType == null) throw new InvalidUseOfNullException("plateType");
		plateType.checkPermission(Permission.WRITE);
		getData().setPlateType(plateType.getData());
	}

	/**
		Get the associated protocol type item. When creating
		{@link PlateEvent} items for a {@link Plate} the protocol for the
		event must be of this type.
		@return The <code>ProtocolType</code> item, or null
			if no protocol type has been specified
		@throws PermissionDeniedException If the logged in user doesn't
			have read permission to the protocol type
		@throws BaseException If there is another error
		@since 3.0
	*/
	public ItemSubtype getProtocolType()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getProtocolType());
	}

	/**
		Set the protocol type of this <code>PlateTypeEvent</code> item.
		When creating {@link PlateEvent} items for a {@link Plate} the 
		protocol for the event must be of this type.

		@param protocolType The new <code>ProtocolType</code>
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission for this event or use permission for the
			protocol type
		@throws BaseException If there is another error
		@since 3.0
	*/
	public void setProtocolType(ItemSubtype protocolType)
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.WRITE);
		if (protocolType != null) 
		{
			protocolType.checkPermission(Permission.USE);
			protocolType.checkItemType(Item.PROTOCOL);
		}
		getData().setProtocolType(protocolType == null ? null : protocolType.getData());
	}

	/**
		Get the order of this event among the list of events.
	*/
	public int getOrdinal()
	{
		return getData().getOrdinal();
	}
	/**
		Set the order of this event in relation to the list
		of all events. The core does not pay any attention to this
		value, it exists mainly for client application to be able to
		display the events in the appropriate order.
		@param ordinal The new ordinal value
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission to this event type
		@throws InvalidDataException If the ordinal is less than one
	*/
	public void setOrdinal(int ordinal)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (ordinal < 1) throw new NumberOutOfRangeException("ordinal", ordinal, 1, false);
		getData().setOrdinal(ordinal);
	}


}
