/*
	$Id: Setting.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.SettingData;


/**
	This is the base class for all setting items. There are
	four types of settings:
	<ul>
	<li>{@link UserClientSetting}: Specific for a user using a client
		application.
	<li>{@link UserDefaultSetting}: Specific for a user, valid for all
		client applications.
	<li>{@link ClientDefaultSetting}: Specific for a client application,
		valid for all users.
	<li>{@link GlobalDefaultSetting}: Valid for all users and client applications.
	</ul>

	@author Nicklas
	@version 2.0
*/
public abstract class Setting<D extends SettingData>
	extends BasicItem<D>
{
	/**
		The type of item represented by this class.
		@see Item#SETTING
		@see #getType()
	*/
	public static final Item TYPE = Item.SETTING;

	/**
		The maximum length of the name that can be stored in the
		database. Check the length against this value before calling the
		{@link #setName(String)} method to avoid exceptions.
	*/
	public static final int MAX_NAME_LENGTH = SettingData.MAX_NAME_LENGTH;

	/**
		The maximum length of the value that can be stored in the
		database. Check the length against this value before calling the
		{@link #setValue(String)} method to avoid exceptions.
	*/
	public static final int MAX_VALUE_LENGTH = SettingData.MAX_VALUE_LENGTH;

	/**
		Get a <code>Setting</code> item when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>Setting</code> item
		@throws ItemNotFoundException If an item with the specified 
			ID is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have {@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static Setting getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Setting s = dc.loadItem(Setting.class, id);
		if (s == null) throw new ItemNotFoundException("Setting[id="+id+"]");
		return s;
	}

	Setting(D settingData)
	{
		super(settingData);
	}

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Add permissions granted to {@link Item#SETTING}.
	*/
	@Override
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		granted |= getSessionControl().getRolePermissions(Item.SETTING);
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/**
		Get the name of this setting.
	 	@return A String object. 
	*/
	public String getName()
	{
		return getData().getName();
	}
	/**
		Set the name for this setting.
	 	@param name Name to be set. 
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If the new name is longer than
			{@link #MAX_NAME_LENGTH}
	*/
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setName(StringUtil.setNotNullString(name, "name", MAX_NAME_LENGTH));
	}

	/**
		Get the value of this setting.
	 	@return A String object. 
	*/
	public String getValue()
	{
		return getData().getValue();
	}
	/**
		Set the value for this setting.
	 	@param value The string value to be set. Can not be longer then {@value #MAX_VALUE_LENGTH} 
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If the new value is longer than
			{@link #MAX_VALUE_LENGTH}
	*/
	public void setValue(String value)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setValue(StringUtil.setNotNullString(value, "value", MAX_VALUE_LENGTH));
	}
}

