/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.Item;

/**
	Chains together one or more source item transformers. Each step in the
	chain should take over from the previous in the sense that the
	{@link SourceItemTransformer#getSourceItemType()} of step N+1
	must match the {@link SourceItemTransformer#getTargetItemType()} 
	of step N.

	@author Nicklas
	@since 3.5
*/
public class SourceItemTransformerChain 
	implements SourceItemTransformer 
{

	private final List<SourceItemTransformer> chain;
	
	private final Item source;
	private final Item target;
	
	/**
		Create a new source item transformer chain. The chain
		must contain at least one element and all steps must have
		matching source and target item types.
		@throws IllegalArgumentException If the chain is null or empty or have non-matching
			transformation steps
	*/
	public SourceItemTransformerChain(List<SourceItemTransformer> chain)
	{
		if (chain == null || chain.size() == 0)
		{
			throw new IllegalArgumentException("Chain is null or empty");
		}
		for (int i = 1; i < chain.size(); i++)
		{
			Item prevTarget = chain.get(i-1).getTargetItemType();
			Item nextSrc = chain.get(i).getSourceItemType();
			if (prevTarget != nextSrc)
			{
				throw new IllegalArgumentException("Source and target mismatch in chain["+i+"]: " + 
					nextSrc.name() + " != " + prevTarget.name());
			}
		}
		this.chain = chain;
		this.source = chain.get(0).getSourceItemType();
		this.target = chain.get(chain.size()-1).getTargetItemType();
	}
	
	/**
		Get the source item type of the first step in the chain.
	*/
	@Override
	public Item getSourceItemType() 
	{
		return source;
	}

	/**
		Get the target item type of the last step in the chain.
	*/
	@Override
	public Item getTargetItemType() 
	{
		return target;
	}
	
	/**
		Transform the source items into target items by invoking each
		step in the transformer chain.
	*/
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		for (SourceItemTransformer sit : chain)
		{
			//System.out.println(sit + " before: " + sit.getSourceItemType() + "=" + source.size() +":" + source);
			source = sit.transform(context, source);
			//System.out.println("After: " + sit.getTargetItemType() + "=" + source.size());
		}
		return source;
	}

	// To test how filters behave when input source set is large
	private Set<Integer> appendFakeSources(Set<Integer> existing)
	{
		Set<Integer> fake = new HashSet<Integer>(existing);
		while (fake.size() < 40000)
		{
			fake.add(-fake.size());
		}
		return fake;
	}
}
