/**
	$Id: JepMapper.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;
import java.util.List;
import org.nfunk.jep.JEP;

import net.sf.basedb.util.jep.Jep;
import net.sf.basedb.util.jep.JepFunction;
import net.sf.basedb.util.jep.LeftFunction;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	A mapper that uses Jep to calculate the final value. It should be used
	for numerical values that needs to be transformed before beeing inserted
	into the database. For example, if a file contains the radius of a spot 
	instead of the diameter, it could be mapped as <code>=2 * col('Radius')</code>.
	<p>
	It can also be used for concatenating string values but syntax with backslashes 
	has better performance. The following mapping expressions are equivalent,
	but we recommend using the first one:
	<pre class="code>
\Name\:\ID\
=col('Name')+':'+col('ID')
=\Name\+':'+\ID\
</pre>
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class JepMapper
	implements Mapper
{

	private final String expression;
	private final JEP parser;
	private final ColFunction colFunction;
	private final LineNoFunction lineNoFunction;
	private final boolean hasLineNo;
	private final DataNoFunction dataNoFunction;
	private final boolean hasDataNo;
	private final NumberFormat numberFormat;
	
	/**
		Create a new JepMapper.
		@param expression The JEP expression
		@param columnHeaders A map from column names to column indexes 
	*/
	public JepMapper(String expression, List<String> columnHeaders)
	{
		this(expression, columnHeaders, null, false);
	}
	
	public JepMapper(String expression, List<String> columnHeaders, NumberFormat numberFormat)
	{
		this(expression, columnHeaders, numberFormat, false);
	}

	public JepMapper(String expression, List<String> columnHeaders, NumberFormat numberFormat,
			boolean ignoreNonExistingColumns)
	{
		this(expression, columnHeaders, numberFormat, ignoreNonExistingColumns, (JepFunction[])null);
	}

	/**
	
		@since 3.1
	*/
	public JepMapper(String expression, List<String> columnHeaders, NumberFormat numberFormat,
		boolean ignoreNonExistingColumns, JepFunction... functions)
	{
		this.expression = expression;
		this.numberFormat = numberFormat;
		this.colFunction = new ColFunction(columnHeaders, numberFormat);
		colFunction.setIgnoreNonExistingColumns(ignoreNonExistingColumns);
		this.lineNoFunction = expression.contains("lineNo()") ? new LineNoFunction() : null;
		this.hasLineNo = lineNoFunction != null;
		this.dataNoFunction = expression.contains("dataNo()") ? new DataNoFunction() : null;
		this.hasDataNo = dataNoFunction != null;
		// Replace: \ColumnName\ with: col('ColumnName')
		expression = expression.replaceAll("\\\\([^\\\\]*)\\\\", "col('$1')");
		
		JepFunction[] extraFunctions = new JepFunction[4 + (functions == null ? 0 : functions.length)];
		extraFunctions[0] = colFunction;
		extraFunctions[1] = lineNoFunction;
		extraFunctions[2] = dataNoFunction;
		extraFunctions[3] = new LeftFunction();
		if (functions != null)
		{
			System.arraycopy(functions, 0, extraFunctions, 4, functions.length);
		}
		this.parser = Jep.newJep(expression, extraFunctions);
	}
	
	
	/*
		From the Mapper interface
		-------------------------------------------
	*/
	@Override
	public String getValue(Data data)
	{
		setData(data);
		// Workaround for bug in JEP error handling 
		// See http://sourceforge.net/tracker/index.php?func=detail&atid=382402&aid=1563324&group_id=24711
		if (parser.hasError()) Jep.clearErrorList(parser);
		Object value = parser.getValueAsObject();
		return value == null ? null : value.toString();
	}
	@Override
	public Integer getInt(Data data)
	{
		setData(data);
		if (parser.hasError()) Jep.clearErrorList(parser);
		return Integer.valueOf((int)parser.getValue());
	}
	@Override
	public Float getFloat(Data data)
	{
		setData(data);
		if (parser.hasError()) Jep.clearErrorList(parser);
		return Float.valueOf((float)parser.getValue());
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "=" + expression;
	}
	// -------------------------------------------

	private void setData(Data data)
	{
		colFunction.setData(data);
		if (hasLineNo) lineNoFunction.setData(data);
		if (hasDataNo) dataNoFunction.setData(data);
	}
}
