/**
	$Id: ResourceBundleFactory.java 5409 2010-09-16 11:32:06Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.resources;


import java.util.Locale;
import java.util.ResourceBundle;

import net.sf.basedb.core.Config;
import net.sf.basedb.core.SessionControl;

/**
	Factory for loading resource bundles. The factory return {@link ResourceBundleWrapper}
	objects which are wrappers around regular {@link ResourceBundle}:s that provide us with
	some extra functionality.
	<p>
	NOTE! Plug-ins and extensions that are not part of the BASE core are usually loaded with a 
	different class loader than the rest of BASE. If they are including resource in their own 
	JAR files, they must use the versions that take a class loader as the last parameter and
	set it to the class loader of of their own classes. Eg. 
	<code>this.getClass().getClassLoader()</code>.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-09-16 13:32:06 +0200 (to, 16 sep 2010) $
*/
public class ResourceBundleFactory
{

	/**
		Same as <code>getResouceBundle(name, sc, null)</code>.
		@see #getResourceBundle(String, SessionControl, ClassLoader)
	*/
	public static ResourceBundleWrapper getResourceBundle(String name, SessionControl sc)
	{
		return getResourceBundle(name, sc, null);
	}
	
	/**
		Get a resource bundle for the given session control. If the session control
		is not null the {@link SessionControl#getLocale()} is used as the locale,
		otherwise the server default locale ({@link Config#getLocale()}) is used.
		
		@param name The name of the resource bundle
		@param sc A session control, or null
		@param loader The class loader to use when locating the resource bundle(s).
			If null, the context class loader of the current thread is used
		@return A resource bundle wrapper
		@see Thread#getContextClassLoader()
	*/
	public static ResourceBundleWrapper getResourceBundle(String name, SessionControl sc, ClassLoader loader)
	{
		Locale locale = sc == null ? Config.getLocale() : sc.getLocale();
		return getResourceBundle(name, locale, loader);
	}

	/**
		Same as <code>getResouceBundle(name, locale, null)</code>.
		@see #getResourceBundle(String, Locale, ClassLoader)
	*/
	public static ResourceBundleWrapper getResourceBundle(String name, Locale locale)
	{
		return getResourceBundle(name, locale, null);
	}
	
	/**
		Get a resource bundle for the given locale.
		
		@param name The name of the resource bundle
		@param locale A locale object
		@param loader The class loader to use when locating the resource bundle(s).
			If null, the context class loader of the current thread is used
		@return A resource bundle wrapper
		@see Thread#getContextClassLoader()
	*/
	public static ResourceBundleWrapper getResourceBundle(String name, Locale locale, ClassLoader loader)
	{
		ResourceBundle.Control control = Utf8PropertiesResourceBundleControl.INSTANCE;
		if (loader == null) loader = Thread.currentThread().getContextClassLoader();
		ResourceBundle bundle = loader == null ?
				ResourceBundle.getBundle(name, locale, control) : 
				ResourceBundle.getBundle(name, locale, loader, control);
		return new ResourceBundleWrapper(name, bundle);
	}
	
	private ResourceBundleFactory()
	{}
	
}
